* Generic session options - page and line size, no page breaks;
OPTIONS nonumber nodate nocenter pagesize=MAX linesize=100 formdlim=' ' spool;                                          
* Kill default titles on output, ODS tables in log, turn on graphics; 
TITLE; ODS TRACE OFF; ODS LISTING; ODS GRAPHICS ON;

********************************************************************************
*******             BEGIN DATA MANIPULATION FOR EXAMPLE 2                *******
*******             CHANGE "filesave" to your directory                  *******
********************************************************************************

* Location for original files for these models  change this path;
%LET filesave=C:\Dropbox\20_PSQF7375_Generalized\PSQF7375_Generalized_Example2; 
LIBNAME filesave "&filesave.";

* Import the STATA version of file into SAS;
PROC IMPORT DATAFILE="&filesave.\Example2.dta" OUT=work.Example2 DBMS=DTA REPLACE; RUN;

* Label variables;
DATA work.Example2; SET work.Example2;  
     LABEL IPV=     "IPV (0=N,1=Y)"
           Neutral= "Aggressive during Neutral Condition (0=N,1=Y)"
           ERcond=  "Condition (1=None, 2=CogR, 3=Supp)" 
           Aggr=     "Aggressive Verbalizations Outcome"; RUN;

* Create labels for groups;
PROC FORMAT; VALUE Fercond 1="1_None" 2="2_CogR" 3="3_Supp"; RUN;

* Add format to data;
DATA work.Example2; SET work.Example2; FORMAT ERcond Fercond.; RUN;

********************************************************************************
******                    BEGIN EXAMPLE 2 MODELS                         *******
********************************************************************************

* Open output directory to save results to;
ODS RTF FILE="&filesave.\Example2_SAS_Output.rtf" STYLE=HTMLBlue STARTPAGE=NO;

TITLE "SAS Overall and Cell Means for Aggressive Verbalizations Outcome";
PROC MEANS NONOBS NDEC=3 DATA=work.Example2;
     CLASS IPV ERcond; WAYS 0 2; VAR Aggr; 
RUN;

TITLE "SAS Histogram for Aggressive Verbalizations Outcome";
PROC UNIVARIATE NOPRINT DATA=work.Example2; VAR Aggr; 
     HISTOGRAM Aggr / MIDPOINTS=0 TO 24 BY 1; 
RUN; QUIT; TITLE;

TITLE "SAS Histogram for Aggressive Verbalizations Outcome";
PROC SGPANEL DATA=work.Example2; 
     PANELBY IPV ERcond / ROWS=2 COLUMNS=3;
     HISTOGRAM Aggr / BINSTART=0 BINWIDTH=1;
     LABEL IPV= "IPV"; 
RUN; QUIT; TITLE1;

TITLE "SAS GLM of Aggressive Verbalizations Outcome using ML";
PROC GLIMMIX DATA=work.Example2 NOCLPRINT GRADIENT NAMELEN=100 METHOD=MSPL PLOTS=(PEARSONPANEL);
CLASS ERcond;
MODEL Aggr = Neutral IPV ERcond IPV*ERcond / SOLUTION CHISQ LINK=IDENTITY DIST=NORMAL;
CONTRAST "Wald Test of Model R2" Neutral 1, IPV 1, ERcond -1 1 0, ERcond -1 0 1, 
                                 IPV*ERcond -1 1 0, IPV*ERcond -1 0 1 / CHISQ; 
* Cell means and simple effects of condition per IPV;
LSMEANS ERcond / ILINK AT (Neutral IPV) = (0 0) CL; * Cell means for IPV=0;
LSMEANS ERcond / ILINK AT (Neutral IPV) = (0 1) CL; * Cell means for IPV=1;
* Simple effects of condition per IPV;
ESTIMATE "None/Cog Difference: IPV=No"   ERcond -1  1  0 IPV*ERcond  0  0  0; 
ESTIMATE "None/Sup Difference: IPV=No"   ERcond -1  0  1 IPV*ERcond  0  0  0; 
ESTIMATE "Cog/Sup  Difference: IPV=No"   ERcond  0 -1  1 IPV*ERcond  0  0  0; 
ESTIMATE "None/Cog Difference: IPV=Yes"  ERcond -1  1  0 IPV*ERcond -1  1  0; 
ESTIMATE "None/Sup Difference: IPV=Yes"  ERcond -1  0  1 IPV*ERcond -1  0  1; 
ESTIMATE "Cog/Sup  Difference: IPV=Yes"  ERcond  0 -1  1 IPV*ERcond  0 -1  1; 
* Simple effects of IPV per condition;
ESTIMATE "No/Yes IPV Difference: No Instruct"      IPV 1 IPV*ERcond  1  0  0; 
ESTIMATE "No/Yes IPV Difference: Cog Reapp"        IPV 1 IPV*ERcond  0  1  0; 
ESTIMATE "No/Yes IPV Difference: Suppression"      IPV 1 IPV*ERcond  0  0  1; 
* Specific interaction contrasts;
ESTIMATE "No/Yes IPV by None/Cog Interaction"            IPV*ERcond -1  1  0; 
ESTIMATE "No/Yes IPV by None/Sup Interaction"            IPV*ERcond -1  0  1; 
ESTIMATE "No/Yes IPV by Cog/Sup  Interaction"            IPV*ERcond  0 -1  1; 
RUN; TITLE;

TITLE "SAS Poisson Model of Aggressive Verbalizations";
PROC GLIMMIX DATA=work.Example2 NOCLPRINT GRADIENT NAMELEN=100 METHOD=MSPL;
CLASS ERcond; 
MODEL Aggr = Neutral IPV ERcond IPV*ERcond / SOLUTION CHISQ LINK=LOG DIST=POISSON;
CONTRAST "Wald Test of Model R2" Neutral 1, IPV 1, ERcond -1 1 0, ERcond -1 0 1, 
                                 IPV*ERcond -1 1 0, IPV*ERcond -1 0 1 / CHISQ; 
* Cell means and simple effects of condition per IPV;
LSMEANS ERcond / ILINK AT (Neutral IPV) = (0 0) CL; * Cell means for IPV=0;
LSMEANS ERcond / ILINK AT (Neutral IPV) = (0 1) CL; * Cell means for IPV=1;
* Simple effects of ERcond per IPV: EXP --> IRRs;
ESTIMATE "None/Cog Difference: IPV=No"   ERcond -1  1  0 IPV*ERcond  0  0  0 / EXP; 
ESTIMATE "None/Sup Difference: IPV=No"   ERcond -1  0  1 IPV*ERcond  0  0  0 / EXP; 
ESTIMATE "Cog/Sup  Difference: IPV=No"   ERcond  0 -1  1 IPV*ERcond  0  0  0 / EXP; 
ESTIMATE "None/Cog Difference: IPV=Yes"  ERcond -1  1  0 IPV*ERcond -1  1  0 / EXP; 
ESTIMATE "None/Sup Difference: IPV=Yes"  ERcond -1  0  1 IPV*ERcond -1  0  1 / EXP; 
ESTIMATE "Cog/Sup  Difference: IPV=Yes"  ERcond  0 -1  1 IPV*ERcond  0 -1  1 / EXP; 
* Simple effects of IPV per condition: EXP --> IRRs;
ESTIMATE "No/Yes IPV Difference: No Instruct"      IPV 1 IPV*ERcond  1  0  0 / EXP; 
ESTIMATE "No/Yes IPV Difference: Cog Reapp"        IPV 1 IPV*ERcond  0  1  0 / EXP; 
ESTIMATE "No/Yes IPV Difference: Suppression"      IPV 1 IPV*ERcond  0  0  1 / EXP; 
* Specific interaction contrasts: EXP --> IRRs;
ESTIMATE "No/Yes IPV by None/Cog Interaction"            IPV*ERcond -1  1  0 / EXP; 
ESTIMATE "No/Yes IPV by None/Sup Interaction"            IPV*ERcond -1  0  1 / EXP; 
ESTIMATE "No/Yes IPV by Cog/Sup  Interaction"            IPV*ERcond  0 -1  1 / EXP; 
RUN; TITLE;

TITLE "SAS Negative Binomial Model of Aggressive Verbalizations";
PROC GLIMMIX DATA=work.Example2 NOCLPRINT GRADIENT NAMELEN=100 METHOD=MSPL;
CLASS ERcond;
MODEL Aggr = Neutral IPV ERcond IPV*ERcond / CHISQ SOLUTION LINK=LOG DIST=NEGBIN;
CONTRAST "Wald Test of Model R2" Neutral 1, IPV 1, ERcond -1 1 0, ERcond -1 0 1, 
                                 IPV*ERcond -1 1 0, IPV*ERcond -1 0 1 / CHISQ; 
* Cell means and simple effects of condition per IPV;
LSMEANS ERcond / ILINK AT (Neutral IPV) = (0 0) CL; * Cell means for IPV=0;
LSMEANS ERcond / ILINK AT (Neutral IPV) = (0 1) CL; * Cell means for IPV=1;
* Simple effects of ERcond per IPV: EXP --> IRRs;
ESTIMATE "None/Cog Difference: IPV=No"   ERcond -1  1  0 IPV*ERcond  0  0  0 / EXP; 
ESTIMATE "None/Sup Difference: IPV=No"   ERcond -1  0  1 IPV*ERcond  0  0  0 / EXP; 
ESTIMATE "Cog/Sup  Difference: IPV=No"   ERcond  0 -1  1 IPV*ERcond  0  0  0 / EXP; 
ESTIMATE "None/Cog Difference: IPV=Yes"  ERcond -1  1  0 IPV*ERcond -1  1  0 / EXP; 
ESTIMATE "None/Sup Difference: IPV=Yes"  ERcond -1  0  1 IPV*ERcond -1  0  1 / EXP; 
ESTIMATE "Cog/Sup  Difference: IPV=Yes"  ERcond  0 -1  1 IPV*ERcond  0 -1  1 / EXP; 
* Simple effects of IPV per condition: EXP --> IRRs;
ESTIMATE "No/Yes IPV Difference: No Instruct"      IPV 1 IPV*ERcond  1  0  0 / EXP; 
ESTIMATE "No/Yes IPV Difference: Cog Reapp"        IPV 1 IPV*ERcond  0  1  0 / EXP; 
ESTIMATE "No/Yes IPV Difference: Suppression"      IPV 1 IPV*ERcond  0  0  1 / EXP; 
* Specific interaction contrasts: EXP --> IRRs;
ESTIMATE "No/Yes IPV by None/Cog Interaction"            IPV*ERcond -1  1  0 / EXP; 
ESTIMATE "No/Yes IPV by None/Sup Interaction"            IPV*ERcond -1  0  1 / EXP; 
ESTIMATE "No/Yes IPV by Cog/Sup  Interaction"            IPV*ERcond  0 -1  1 / EXP; 
OUTPUT OUT=work.NegBinPred PREDICTED(ILINK)=PredCount; * Save predicted counts to dataset;
RUN; TITLE;
PROC CORR NOSIMPLE DATA=work.NegBinPred; VAR Aggr; WITH PredCount; RUN; * Get corr of pred with Aggr;

TITLE "Scatterplot of Predicted and Actual Outcomes";
PROC SGPLOT DATA=work.NegBinPred;
	SCATTER x=PredCount y=Aggr;
	REG     x=PredCount y=Aggr;
RUN;

TITLE "SAS Zero-Inflated Poisson Model of Aggressive Verbalizations";
PROC GENMOD DATA=work.Example2 NAMELEN=100;
CLASS ERcond; 
MODEL Aggr = Neutral IPV ERcond IPV*ERcond / LINK=LOG DIST=ZIP;
ZEROMODEL / LINK=LOGIT; 
RUN; TITLE;

TITLE "SAS Zero-Inflated Negative Binomial Model of Aggressive Verbalizations";
PROC GENMOD DATA=work.Example2 NAMELEN=100;
CLASS ERcond; 
MODEL Aggr = Neutral IPV ERcond IPV*ERcond / LINK=LOG DIST=ZINB;
ZEROMODEL / LINK=LOGIT; 
RUN; TITLE;

TITLE "SAS Heterogeneous Negative Binomial Model of Aggressive Verbalizations";
PROC NLMIXED DATA=work.Example2 METHOD=GAUSS TECH=QUANEW GCONV=1e-12;
* Must list all parms to be estimated here with start values;
* Bs = fixed effects predicting the mean;
* Ds = fixed effects predicting the dispersion (scale factor);
PARMS B0 0.85 B1neutral  0.46 B2IPV  0.59 B3SvN  0.12 B4SvC -0.97 B5IPVxSvN -0.53 B6IPVxSvC -2.42
      D0 0.54 D1neutral -0.68 D2IPV -0.16 D3SvN -0.30 D4SvC  0.63 D5IPVxSvN -0.55 D6IPVxSvC -12.29;
 * Linear predictor for the mean;
   LogCount = B0 + B1neutral*neutral + B2IPV*IPV + B3SvN*(ERcond=1) + B4SvC*(ERcond=2) 
              + B5IPVxSvN*IPV*(ERcond=1) + B6IPVxSvC*IPV*(ERcond=2);
   ExpCount = EXP(LogCount); * Inverse link;
 * Linear predictor for the dispersion scale parameter;
   LogScale = D0 + D1neutral*neutral + D2IPV*IPV + D3SvN*(ERcond=1) + D4SvC*(ERcond=2) 
              + D5IPVxSvN*IPV*(ERcond=1) + D6IPVxSvC*IPV*(ERcond=2);
   ExpScale=EXP(LogScale); * Inverse Link;
 * Log-likelihood for NB-1 (constant) or NB-2 (multiplicative);
   * NB-1: LL = Aggr*LOG(ExpScale) - (Aggr+(1/(ExpScale/ExpCount)))*LOG(1+ExpScale)
           + lgamma(Aggr+(1/(ExpScale/ExpCount))) - lgamma(1/(ExpScale/ExpCount)) - lgamma(Aggr+1);
   * NB-2; LL = Aggr*LOG(ExpScale*ExpCount) - (Aggr+(1/ExpScale))*LOG(1+(ExpScale*ExpCount))
           + lgamma(Aggr+(1/ExpScale)) - lgamma(1/ExpScale) - lgamma(Aggr+1);
MODEL Aggr ~ GENERAL(LL);
RUN;

* Close word file;
ODS RTF CLOSE;
