* Prevent "more" messages from appearing
set more off
* Control line length
set linesize 150

********************************************************************************
*******             BEGIN DATA MANIPULATION FOR EXAMPLE 2               *******
*******             CHANGE "filesave" to your directory                  *******
********************************************************************************

* Defining global variable for file location to be replaced in code below
global filesave "C:\Dropbox\20_PSQF7375_Generalized\PSQF7375_Generalized_Example2"

* Import data, transform variables, create and apply value formats
use "$filesave\Example2.dta", clear
label define fercond 1 "1_None" 2 "2_CogR" 3 "3_Supp"
label values ercond fercond

********************************************************************************
******                    BEGIN EXAMPLE 2 MODELS						 *******
********************************************************************************

* Save results to separate file
log using $filesave\Example2_STATA_Output.log, replace name(Example2)

display as result "STATA Overall and Cell Means for Aggressive Verbalizations Outcome"
format aggr %8.3f
tabulate ipv ercond, summarize(aggr)
 
display as result "STATA Histogram for Aggressive Verbalizations Outcome"
hist aggr, percent normal discrete width(1) start(0) 


display as result "STATA GLM of Aggressive Verbalizations Outcome using ML"
glm aggr c.neutral c.ipv##ib(last).ercond, ///
         link(identity) family(gaussian) // normal residuals 
estat ic, n(225),
* Wald Test of Model R2 (not given by default in GLM)
test (c.neutral=0)(c.ipv=0)(i1.ercond=i2.ercond)(i1.ercond=i3.ercond) ///
     (c.ipv#i1.ercond=c.ipv#i2.ercond)(c.ipv#i1.ercond=c.ipv#i3.ercond)
* Wald Test of Omnibus main effect of group and interaction with IPV
contrast  i.ercond,        // DF=2 test of main effect at IPV=0
contrast  c.ipv#i.ercond,  // DF=2 test of interaction
* Cell means of condition per IPV
margins i.ercond, at(c.neutral=0 c.ipv=0) // cell means for IPV=0
margins i.ercond, at(c.neutral=0 c.ipv=1) // cell means for IPV=1
* Simple effects of condition per IPV
lincom i1.ercond*-1 + i2.ercond*1 + c.ipv#i1.ercond*0  + c.ipv#i2.ercond*0 // N/C: IPV=No
lincom i1.ercond*-1 + i3.ercond*1 + c.ipv#i1.ercond*0  + c.ipv#i3.ercond*0 // N/S: IPV=No
lincom i2.ercond*-1 + i3.ercond*1 + c.ipv#i2.ercond*0  + c.ipv#i3.ercond*0 // C/S: IPV=No
lincom i1.ercond*-1 + i2.ercond*1 + c.ipv#i1.ercond*-1 + c.ipv#i2.ercond*1 // N/C: IPV=Yes
lincom i1.ercond*-1 + i3.ercond*1 + c.ipv#i1.ercond*-1 + c.ipv#i3.ercond*1 // N/S: IPV=Yes
lincom i2.ercond*-1 + i3.ercond*1 + c.ipv#i2.ercond*-1 + c.ipv#i3.ercond*1 // C/S: IPV=Yes
* Simple effects of IPV per condition
lincom c.ipv + c.ipv#i1.ercond  // No/Yes IPV Difference: No Instruction
lincom c.ipv + c.ipv#i2.ercond  // No/Yes IPV Difference: Cognitive Reappraisal
lincom c.ipv + c.ipv#i3.ercond  // No/Yes IPV Difference: Suppression
* Specific interaction contrasts
contrast {c.ipv#i.ercond -1  1  0}, pveffects // No/Yes IPV by None/Cog Interaction
contrast {c.ipv#i.ercond -1  0  1}, pveffects // No/Yes IPV by None/Sup Interaction
contrast {c.ipv#i.ercond  0 -1  1}, pveffects // No/Yes IPV by Cog/Sup  Interaction


display as result "STATA Poisson Model of Aggressive Verbalizations Outcome"
glm aggr c.neutral c.ipv##ib(last).ercond, ///
         link(log) family(poisson) // Poisson residuals 
estat ic, n(225),
* Wald Test of Model R2 (not given by default in GLM)
test (c.neutral=0)(c.ipv=0)(i1.ercond=i2.ercond)(i1.ercond=i3.ercond) ///
     (c.ipv#i1.ercond=c.ipv#i2.ercond)(c.ipv#i1.ercond=c.ipv#i3.ercond)
* Wald Test of Omnibus main effect of group and interaction with IPV
contrast  i.ercond,        // DF=2 test of main effect at IPV=0
contrast  c.ipv#i.ercond,  // DF=2 test of interaction
* Cell means of condition per IPV
margins i.ercond, at(c.neutral=0 c.ipv=0) predict(xb) // log count cell means for IPV=0
margins i.ercond, at(c.neutral=0 c.ipv=1) predict(xb) // log count cell means for IPV=1
margins i.ercond, at(c.neutral=0 c.ipv=0) // count cell means for IPV=0
margins i.ercond, at(c.neutral=0 c.ipv=1) // count cell means for IPV=1
* Simple effects of condition per IPV
lincom i1.ercond*-1 + i2.ercond*1 + c.ipv#i1.ercond*0  + c.ipv#i2.ercond*0 // N/C: IPV=No
lincom i1.ercond*-1 + i3.ercond*1 + c.ipv#i1.ercond*0  + c.ipv#i3.ercond*0 // N/S: IPV=No
lincom i2.ercond*-1 + i3.ercond*1 + c.ipv#i2.ercond*0  + c.ipv#i3.ercond*0 // C/S: IPV=No
lincom i1.ercond*-1 + i2.ercond*1 + c.ipv#i1.ercond*-1 + c.ipv#i2.ercond*1 // N/C: IPV=Yes
lincom i1.ercond*-1 + i3.ercond*1 + c.ipv#i1.ercond*-1 + c.ipv#i3.ercond*1 // N/S: IPV=Yes
lincom i2.ercond*-1 + i3.ercond*1 + c.ipv#i2.ercond*-1 + c.ipv#i3.ercond*1 // C/S: IPV=Yes
* Simple effects of IPV per condition
lincom c.ipv + c.ipv#i1.ercond  // No/Yes IPV Difference: No Instruction
lincom c.ipv + c.ipv#i2.ercond  // No/Yes IPV Difference: Cognitive Reappraisal
lincom c.ipv + c.ipv#i3.ercond  // No/Yes IPV Difference: Suppression
* Specific interaction contrasts
contrast {c.ipv#i.ercond -1  1  0}, pveffects // No/Yes IPV by None/Cog Interaction
contrast {c.ipv#i.ercond -1  0  1}, pveffects // No/Yes IPV by None/Sup Interaction
contrast {c.ipv#i.ercond  0 -1  1}, pveffects // No/Yes IPV by Cog/Sup  Interaction

display as result "STATA Poisson Model of Aggressive Verbalizations Outcome"
display as result "Request Incidence-Rate Ratios (via eform or irr)"
glm aggr c.neutral c.ipv##ib(last).ercond, eform ///
         link(log) family(poisson) // Poisson residuals 
* Simple effects of condition per IPV
lincom i1.ercond*-1 + i2.ercond*1 + c.ipv#i1.ercond*0  + c.ipv#i2.ercond*0, irr // N/C: IPV=No
lincom i1.ercond*-1 + i3.ercond*1 + c.ipv#i1.ercond*0  + c.ipv#i3.ercond*0, irr // N/S: IPV=No
lincom i2.ercond*-1 + i3.ercond*1 + c.ipv#i2.ercond*0  + c.ipv#i3.ercond*0, irr // C/S: IPV=No
lincom i1.ercond*-1 + i2.ercond*1 + c.ipv#i1.ercond*-1 + c.ipv#i2.ercond*1, irr // N/C: IPV=Yes
lincom i1.ercond*-1 + i3.ercond*1 + c.ipv#i1.ercond*-1 + c.ipv#i3.ercond*1, irr // N/S: IPV=Yes
lincom i2.ercond*-1 + i3.ercond*1 + c.ipv#i2.ercond*-1 + c.ipv#i3.ercond*1, irr // C/S: IPV=Yes
* Simple effects of IPV per condition
lincom c.ipv + c.ipv#i1.ercond, irr  // No/Yes IPV Difference: No Instruction
lincom c.ipv + c.ipv#i2.ercond, irr  // No/Yes IPV Difference: Cognitive Reappraisal
lincom c.ipv + c.ipv#i3.ercond, irr  // No/Yes IPV Difference: Suppression
* Specific interaction contrasts
contrast {c.ipv#i.ercond -1  1  0}, pveffects eform // No/Yes IPV by None/Cog Interaction
contrast {c.ipv#i.ercond -1  0  1}, pveffects eform // No/Yes IPV by None/Sup Interaction
contrast {c.ipv#i.ercond  0 -1  1}, pveffects eform // No/Yes IPV by Cog/Sup  Interaction


display as result "STATA Negative Binomial Model of Aggressive Verbalizations Outcome"
nbreg aggr c.neutral c.ipv##ib(last).ercond,  // Run this to get LR test of scale factor

display as result "STATA Negative Binomial Model of Aggressive Verbalizations Outcome"
glm aggr c.neutral c.ipv##ib(last).ercond, ///
         link(log) family(nbinomial ml) // Negative Binomial residuals 
estat ic, n(225),
* Wald Test of Model R2 (not given by default in GLM)
test (c.neutral=0)(c.ipv=0)(i1.ercond=i2.ercond)(i1.ercond=i3.ercond) ///
     (c.ipv#i1.ercond=c.ipv#i2.ercond)(c.ipv#i1.ercond=c.ipv#i3.ercond)
* Wald Test of Omnibus main effect of group and interaction with IPV
contrast  i.ercond,        // DF=2 test of main effect at IPV=0
contrast  c.ipv#i.ercond,  // DF=2 test of interaction
* Cell means of condition per IPV
margins i.ercond, at(c.neutral=0 c.ipv=0) predict(xb) // log count cell means for IPV=0
margins i.ercond, at(c.neutral=0 c.ipv=1) predict(xb) // log count cell means for IPV=1
margins i.ercond, at(c.neutral=0 c.ipv=0) // count cell means for IPV=0
margins i.ercond, at(c.neutral=0 c.ipv=1) // count cell means for IPV=1
* Simple effects of condition per IPV
lincom i1.ercond*-1 + i2.ercond*1 + c.ipv#i1.ercond*0  + c.ipv#i2.ercond*0 // N/C: IPV=No
lincom i1.ercond*-1 + i3.ercond*1 + c.ipv#i1.ercond*0  + c.ipv#i3.ercond*0 // N/S: IPV=No
lincom i2.ercond*-1 + i3.ercond*1 + c.ipv#i2.ercond*0  + c.ipv#i3.ercond*0 // C/S: IPV=No
lincom i1.ercond*-1 + i2.ercond*1 + c.ipv#i1.ercond*-1 + c.ipv#i2.ercond*1 // N/C: IPV=Yes
lincom i1.ercond*-1 + i3.ercond*1 + c.ipv#i1.ercond*-1 + c.ipv#i3.ercond*1 // N/S: IPV=Yes
lincom i2.ercond*-1 + i3.ercond*1 + c.ipv#i2.ercond*-1 + c.ipv#i3.ercond*1 // C/S: IPV=Yes
* Simple effects of IPV per condition
lincom c.ipv + c.ipv#i1.ercond  // No/Yes IPV Difference: No Instruction
lincom c.ipv + c.ipv#i2.ercond  // No/Yes IPV Difference: Cognitive Reappraisal
lincom c.ipv + c.ipv#i3.ercond  // No/Yes IPV Difference: Suppression
* Specific interaction contrasts
contrast {c.ipv#i.ercond -1  1  0}, pveffects // No/Yes IPV by None/Cog Interaction
contrast {c.ipv#i.ercond -1  0  1}, pveffects // No/Yes IPV by None/Sup Interaction
contrast {c.ipv#i.ercond  0 -1  1}, pveffects // No/Yes IPV by Cog/Sup  Interaction
 
display as result "STATA Negative Binomial Model of Aggressive Verbalizations Outcome"
display as result "Request Incidence-Rate Ratios (via eform or irr)"
glm aggr c.neutral c.ipv##ib(last).ercond, eform ///
         link(log) family(nbinomial ml) // Negative binomial residuals 
* Simple effects of condition per IPV
lincom i1.ercond*-1 + i2.ercond*1 + c.ipv#i1.ercond*0  + c.ipv#i2.ercond*0, irr // N/C: IPV=No
lincom i1.ercond*-1 + i3.ercond*1 + c.ipv#i1.ercond*0  + c.ipv#i3.ercond*0, irr // N/S: IPV=No
lincom i2.ercond*-1 + i3.ercond*1 + c.ipv#i2.ercond*0  + c.ipv#i3.ercond*0, irr // C/S: IPV=No
lincom i1.ercond*-1 + i2.ercond*1 + c.ipv#i1.ercond*-1 + c.ipv#i2.ercond*1, irr // N/C: IPV=Yes
lincom i1.ercond*-1 + i3.ercond*1 + c.ipv#i1.ercond*-1 + c.ipv#i3.ercond*1, irr // N/S: IPV=Yes
lincom i2.ercond*-1 + i3.ercond*1 + c.ipv#i2.ercond*-1 + c.ipv#i3.ercond*1, irr // C/S: IPV=Yes
* Simple effects of IPV per condition
lincom c.ipv + c.ipv#i1.ercond, irr  // No/Yes IPV Difference: No Instruction
lincom c.ipv + c.ipv#i2.ercond, irr  // No/Yes IPV Difference: Cognitive Reappraisal
lincom c.ipv + c.ipv#i3.ercond, irr  // No/Yes IPV Difference: Suppression
* Specific interaction contrasts
contrast {c.ipv#i.ercond -1  1  0}, pveffects eform // No/Yes IPV by None/Cog Interaction
contrast {c.ipv#i.ercond -1  0  1}, pveffects eform // No/Yes IPV by None/Sup Interaction
contrast {c.ipv#i.ercond  0 -1  1}, pveffects eform // No/Yes IPV by Cog/Sup  Interaction
 
 
display as result "STATA Zero-Inflated Poisson Model of Aggressive Verbalizations Outcome"
zip aggr c.neutral c.ipv##ib(last).ercond, inflate(_cons), // inflate=zero-inflation model
estat ic, n(225),
	
*** STATA BLEW UP!!
*display as result "Zero-Inflated Negative Binomial of Aggressive Verbalizations Outcome"
*zinb aggr c.neutral c.ipv##ib(last).ercond, inflate(_cons), // inflate=zero-inflation model
*estat ic, n(225),

display as result "STATA Heterogeneous Negative Binomial of Aggressive Verbalizations Outcome""
gnbreg aggr c.neutral c.ipv##ib(last).ercond, lnalpha(neutral c.ipv##ib(last).ercond)
estat ic, n(225),

* Close log	
log close Example2
